@@ -17,11 +17,25 @@
 #  under the License.
 
 """setup for multi product plugin"""
+import sys, codecs
+from pkg_resources import parse_version
 from setuptools import setup
 
+
+# Force UTF-8 for stdout/err if --utf8 option is specified.
+# For some reason python doesn't obey LANG/LC_CTYPE settings
+# if output is not a terminal (e.g. pipes don't work).
+if __name__ == '__main__':
+    for ac,av in enumerate(sys.argv):
+        if av == '--utf8':
+            sys.stdout = codecs.getwriter('utf8')(sys.stdout)
+            sys.stderr = codecs.getwriter('utf8')(sys.stderr)
+            del sys.argv[ac]
+            break
+
 setup(
     name = 'BloodhoundMultiProduct',
-    version = '0.6.0',
+    version = '0.7.0',
     description = "Multiproduct support for Apache(TM) Bloodhound.",
     author = "Apache Bloodhound",
     license = "Apache License v2",
@@ -32,10 +46,12 @@ setup(
             'multiproduct.model = multiproduct.model',
             'multiproduct.perm = multiproduct.perm',
             'multiproduct.product_admin = multiproduct.product_admin',
+            'multiproduct.ticket.batch = multiproduct.ticket.batch',
             'multiproduct.ticket.query = multiproduct.ticket.query',
             'multiproduct.ticket.web_ui = multiproduct.ticket.web_ui',
             'multiproduct.web_ui = multiproduct.web_ui',
         ],},
     test_suite='tests.test_suite',
+    tests_require=['unittest2' if parse_version(sys.version) < parse_version('2.7') else '']
 )
 
