@@ -42,6 +42,7 @@ from trac.tests.env import EnvironmentTestCase
 from trac.ticket.report import ReportModule
 from trac.ticket.web_ui import TicketModule
 from trac.util.text import to_unicode
+from trac.web.href import Href
 
 from multiproduct.api import MultiProductSystem
 from multiproduct.env import ProductEnvironment
@@ -233,7 +234,7 @@ class MultiproductTestCase(unittest.TestCase):
         except OperationalError:
             # Database is upgraded, but database version was deleted.
             # Complete the upgrade by inserting default product.
-            self.mpsystem._insert_default_product(env.db_transaction)
+            mpsystem._insert_default_product(env.db_transaction)
         # assume that the database schema has been upgraded, enable
         # multi-product schema support in environment
         env.enable_multiproduct_schema(True)
@@ -485,6 +486,18 @@ class ProductEnvApiTestCase(MultiproductTestCase):
         # Unregister class C as its fake module might break something else.
         ComponentMeta._components.remove(C)
 
+    def test_href_is_lazy(self):
+        href = self.product_env.href
+        self.assertIs(href, self.product_env.href)
+
+    def test_abs_href_is_lazy(self):
+        abs_href = self.product_env.abs_href
+        self.assertIs(abs_href, self.product_env.abs_href)
+
+    def test_path_is_lazy(self):
+        path = self.product_env.path
+        self.assertIs(path, self.product_env.path)
+
     def test_path(self):
         """Testing env.path"""
         self.assertEqual(self.product_env.path,
@@ -558,9 +571,11 @@ class ProductEnvHrefTestCase(MultiproductTestCase):
     def setUp(self):
         self._mp_setup()
         self.env.path = '/path/to/env'
+        self.env.abs_href = Href('http://globalenv.com/trac.cgi')
         url_pattern = getattr(getattr(self, self._testMethodName).im_func,
                               'product_base_url', '')
         self.env.config.set('multiproduct', 'product_base_url', url_pattern)
+        self.env.config.set('trac', 'base_url', 'http://globalenv.com/trac.cgi')
         self.product_env = ProductEnvironment(self.env, self.default_product)
 
     def tearDown(self):
@@ -579,35 +594,93 @@ class ProductEnvHrefTestCase(MultiproductTestCase):
     def test_href_subdomain(self):
         """Test product sub domain base URL
         """
+        self.assertEqual('/', self.product_env.href())
         self.assertEqual('http://tp1.domain.tld', self.product_env.abs_href())
 
     @product_base_url('/path/to/bloodhound/$(prefix)s')
     def test_href_sibling_paths(self):
         """Test product base URL at sibling paths
         """
-        self.assertEqual('http://example.org/trac.cgi/path/to/bloodhound/tp1', 
+        self.assertEqual('/trac.cgi/path/to/bloodhound/tp1',
+                         self.product_env.href())
+        self.assertEqual('http://globalenv.com/trac.cgi/path/to/bloodhound/tp1',
                          self.product_env.abs_href())
 
     @product_base_url('/$(envname)s/$(prefix)s')
     def test_href_inherit_sibling_paths(self):
         """Test product base URL at sibling paths inheriting configuration.
         """
-        self.assertEqual('http://example.org/trac.cgi/env/tp1', 
+        self.assertEqual('/trac.cgi/env/tp1', self.product_env.href())
+        self.assertEqual('http://globalenv.com/trac.cgi/env/tp1',
+                         self.product_env.abs_href())
+
+    @product_base_url('')
+    def test_href_default(self):
+        """Test product base URL is to a default
+        """
+        self.assertEqual('/trac.cgi/products/tp1', self.product_env.href())
+        self.assertEqual('http://globalenv.com/trac.cgi/products/tp1',
                          self.product_env.abs_href())
 
     @product_base_url('/products/$(prefix)s')
     def test_href_embed(self):
         """Test default product base URL /products/prefix
         """
-        self.assertEqual('http://example.org/trac.cgi/products/tp1', 
+        self.assertEqual('/trac.cgi/products/tp1', self.product_env.href())
+        self.assertEqual('http://globalenv.com/trac.cgi/products/tp1',
                          self.product_env.abs_href())
 
     @product_base_url('http://$(envname)s.tld/bh/$(prefix)s')
     def test_href_complex(self):
         """Test complex product base URL
         """
+        self.assertEqual('/bh/tp1', self.product_env.href())
         self.assertEqual('http://env.tld/bh/tp1', self.product_env.abs_href())
 
+    @product_base_url('http://$(prefix)s.$(envname)s.tld/')
+    def test_product_href_uses_multiproduct_product_base_url(self):
+        """Test that [multiproduct] product_base_url is used to compute
+        abs_href for the product environment when [trac] base_url for
+        the product environment is an empty string (the default).
+        """
+        # Global URLs
+        self.assertEqual('http://globalenv.com/trac.cgi', self.env.base_url)
+        self.assertEqual('/trac.cgi', self.env.href())
+        self.assertEqual('http://globalenv.com/trac.cgi', self.env.abs_href())
+
+        # Product URLs
+        self.assertEqual('', self.product_env.base_url)
+        self.assertEqual('/', self.product_env.href())
+        self.assertEqual('http://tp1.env.tld', self.product_env.abs_href())
+
+    @product_base_url('http://$(prefix)s.$(envname)s.tld/')
+    def test_product_href_uses_products_base_url(self):
+        """Test that [trac] base_url for the product environment is used to
+        compute abs_href for the product environment when [trac] base_url
+        for the product environment is different than [trac] base_url for
+        the global environment.
+        """
+        self.product_env.config.set('trac', 'base_url', 'http://productenv.com')
+        self.product_env.config.save()
+
+        self.assertEqual('http://productenv.com', self.product_env.base_url)
+        self.assertEqual('/', self.product_env.href())
+        self.assertEqual('http://productenv.com', self.product_env.abs_href())
+
+    @product_base_url('http://$(prefix)s.$(envname)s.tld/')
+    def test_product_href_global_and_product_base_urls_same(self):
+        """Test that [multiproduct] product_base_url is used to compute
+        abs_href for the product environment when [trac] base_url is the same
+        for the product and global environment.
+        """
+        self.product_env.config.set('trac', 'base_url',
+                                    self.env.config.get('trac', 'base_url'))
+        self.product_env.config.save()
+
+        self.assertEqual('', self.product_env.base_url)
+        self.assertEqual('/', self.product_env.href())
+        self.assertEqual('http://tp1.env.tld', self.product_env.abs_href())
+
     product_base_url = staticmethod(product_base_url)
 
 
