@@ -31,20 +31,28 @@ from genshi.builder import tag
 from genshi import HTML
 from trac.perm import IPermissionRequestor
 from trac.search import shorten_result
-from trac.config import OrderedExtensionsOption, ListOption, Option
+from trac.config import OrderedExtensionsOption, ListOption, Option, BoolOption
 from trac.util.presentation import Paginator
 from trac.util.datefmt import format_datetime, user_time
-from trac.web import IRequestHandler
+from trac.web import IRequestHandler, IRequestFilter
 from trac.util.translation import _
 from trac.util.html import find_element
-from trac.web.chrome import (INavigationContributor, ITemplateProvider,
-                             add_link, add_stylesheet, web_context)
+from trac.web.chrome import (ITemplateProvider,
+                             add_link, add_stylesheet, prevnext_nav,
+                             web_context)
 from bhsearch.api import (BloodhoundSearchApi, ISearchParticipant, SCORE, ASC,
                           DESC, IndexFields, SortInstruction)
+from bhsearch.utils import get_global_env, using_multiproduct
 from trac.wiki.formatter import extract_link
+from multiproduct.env import ProductEnvironment
+from multiproduct.web_ui import ProductModule
 
 SEARCH_PERMISSION = 'SEARCH_VIEW'
 DEFAULT_RESULTS_PER_PAGE = 10
+SEARCH_URL = '/search'
+BHSEARCH_URL = '/bhsearch'
+SEARCH_URLS_RE = re.compile(r'/(?P<prefix>bh)?search(?P<suffix>.*)')
+
 
 class RequestParameters(object):
     """
@@ -61,9 +69,13 @@ class RequestParameters(object):
     VIEW = "view"
     SORT = "sort"
     DEBUG = "debug"
+    PRODUCT = "product_prefix"
+    PRODUCT_ID = "productid"
 
-    def __init__(self, req):
+    def __init__(self, req, href=None):
+        # pylint: disable=too-many-branches
         self.req = req
+        self.href = href or req.href
 
         self.original_query = req.args.getfirst(self.QUERY)
         if self.original_query is None:
@@ -86,6 +98,8 @@ class RequestParameters(object):
             DEFAULT_RESULTS_PER_PAGE))
         self.page = int(req.args.getfirst(self.PAGE, '1'))
         self.type = req.args.getfirst(self.TYPE)
+
+        self.product = req.args.getfirst(self.PRODUCT)
         self.debug = int(req.args.getfirst(self.DEBUG, '0'))
 
         self.params = {
@@ -104,6 +118,8 @@ class RequestParameters(object):
             self.params[self.PAGE] = self.page
         if self.type:
             self.params[self.TYPE] = self.type
+        if self.product is not None:
+            self.params[self.PRODUCT] = self.product
         if self.filter_queries:
             self.params[RequestParameters.FILTER_QUERY] = self.filter_queries
         if self.sort_string:
@@ -132,8 +148,6 @@ class RequestParameters(object):
 
         return sort if sort else None
 
-
-
     def _remove_possible_duplications(self, parameters_list):
         seen = set()
         return [parameter for parameter in parameters_list
@@ -150,8 +164,13 @@ class RequestParameters(object):
             view=None,
             skip_view=False,
             sort=None,
-            skip_sort = False,
+            skip_sort=False,
+            query=None,
+            skip_query=False,
+            product=None,
+            skip_product=False,
     ):
+        # pylint: disable=too-many-locals,too-many-branches
         params = copy.deepcopy(self.params)
 
         if skip_sort:
@@ -183,7 +202,17 @@ class RequestParameters(object):
         elif force_filters is not None:
             params[self.FILTER_QUERY] = force_filters
 
-        return self.req.href.bhsearch(**params)
+        if skip_query:
+            self._delete_if_exists(params, self.QUERY)
+        elif query is not None:
+            params[self.QUERY] = query
+
+        if skip_product:
+            self._delete_if_exists(params, self.PRODUCT)
+        elif product is not None:
+            params[self.PRODUCT] = product
+
+        return self.href.bhsearch(**params)
 
     def _create_sort_expression(self, sort):
         """
@@ -204,10 +233,11 @@ class RequestParameters(object):
         if name in params:
             del params[name]
 
+
 class BloodhoundSearchModule(Component):
     """Main search page"""
-    implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
-        ITemplateProvider,
+    implements(IPermissionRequestor, IRequestHandler,
+        ITemplateProvider, IRequestFilter
         #           IWikiSyntaxProvider #todo: implement later
     )
 
@@ -220,6 +250,7 @@ class BloodhoundSearchModule(Component):
 
     prefix = "all"
     default_grid_fields = [
+        IndexFields.PRODUCT,
         IndexFields.ID,
         IndexFields.TYPE,
         IndexFields.TIME,
@@ -230,7 +261,7 @@ class BloodhoundSearchModule(Component):
     default_facets = ListOption(
         BHSEARCH_CONFIG_SECTION,
         prefix + '_default_facets',
-        default=",".join([IndexFields.TYPE]),
+        default=",".join([IndexFields.PRODUCT, IndexFields.TYPE]),
         doc="""Default facets applied to search view of all resources""")
 
     default_view = Option(
@@ -245,17 +276,31 @@ class BloodhoundSearchModule(Component):
         default=",".join(default_grid_fields),
         doc="""Default fields for grid view for specific resource""")
 
+    default_search = BoolOption(
+        BHSEARCH_CONFIG_SECTION,
+        'is_default',
+        default=False,
+        doc="""Searching from quicksearch uses bhsearch.""")
 
-    # INavigationContributor methods
-    def get_active_navigation_item(self, req):
-        # pylint: disable=unused-argument
-        return 'bhsearch'
+    redirect_enabled = BoolOption(
+        BHSEARCH_CONFIG_SECTION,
+        'enable_redirect',
+        default=False,
+        doc="""Redirect links pointing to trac search to bhsearch""")
 
-    def get_navigation_items(self, req):
-        if SEARCH_PERMISSION in req.perm:
-            yield ('mainnav', 'bhsearch',
-                   tag.a(_('Bloodhound Search'), href=self.env.href.bhsearch())
-                )
+    global_quicksearch = BoolOption(
+        BHSEARCH_CONFIG_SECTION,
+        'global_quicksearch',
+        default=True,
+        doc="""Quicksearch searches all products, even when used
+            in product env.""")
+
+    query_suggestions_enabled = BoolOption(
+        BHSEARCH_CONFIG_SECTION,
+        'query_suggestions',
+        default=True,
+        doc="""Display query suggestions."""
+    )
 
     # IPermissionRequestor methods
     def get_permission_actions(self):
@@ -263,19 +308,29 @@ class BloodhoundSearchModule(Component):
 
     # IRequestHandler methods
     def match_request(self, req):
-        return re.match(r'/bhsearch?', req.path_info) is not None
+        return re.match('^%s' % BHSEARCH_URL, req.path_info) is not None
 
     def process_request(self, req):
         req.perm.assert_permission(SEARCH_PERMISSION)
+
+        if self._is_opensearch_request(req):
+            return ('opensearch.xml', {},
+                    'application/opensearchdescription+xml')
+
         request_context = RequestContext(
             self.env,
             req,
             self.search_participants,
             self.default_view,
             self.all_grid_fields,
-            self.default_facets
+            self.default_facets,
+            self.global_quicksearch,
+            self.query_suggestions_enabled,
         )
 
+        if request_context.requires_redirect:
+            req.redirect(request_context.parameters.create_href(), True)
+
         # compatibility with legacy search
         req.search_query = request_context.parameters.query
 
@@ -294,6 +349,9 @@ class BloodhoundSearchModule(Component):
         request_context.process_results(query_result)
         return self._return_data(req, request_context.data)
 
+    def _is_opensearch_request(self, req):
+        return req.path_info == BHSEARCH_URL + '/opensearch'
+
     def _return_data(self, req, data):
         add_stylesheet(req, 'common/css/search.css')
         return 'bhsearch.html', data, None
@@ -307,10 +365,33 @@ class BloodhoundSearchModule(Component):
     def get_templates_dirs(self):
         return [pkg_resources.resource_filename(__name__, 'templates')]
 
+    # IRequestFilter methods
+    def pre_process_request(self, req, handler):
+        if SEARCH_URLS_RE.match(req.path_info):
+            if self.redirect_enabled:
+                return self
+        return handler
+
+    def post_process_request(self, req, template, data, content_type):
+        if data is None:
+            return template, data, content_type
+
+        if self.redirect_enabled:
+            data['search_handler'] = req.href.bhsearch()
+        elif req.path_info.startswith(SEARCH_URL):
+            data['search_handler'] = req.href.search()
+        elif self.default_search or req.path_info.startswith(BHSEARCH_URL):
+            data['search_handler'] = req.href.bhsearch()
+        else:
+            data['search_handler'] = req.href.search()
+        return template, data, content_type
+
+
 class RequestContext(object):
     DATA_ACTIVE_FILTER_QUERIES = 'active_filter_queries'
+    DATA_ACTIVE_PRODUCT = 'active_product'
+    DATA_ACTIVE_QUERY = 'active_query'
     DATA_BREADCRUMBS_TEMPLATE = 'resourcepath_template'
-    DATA_TYPES = "types"
     DATA_HEADERS = "headers"
     DATA_ALL_VIEWS = "all_views"
     DATA_VIEW = "view"
@@ -319,15 +400,17 @@ class RequestContext(object):
     DATA_DEBUG = 'debug'
     DATA_PAGE_HREF = 'page_href'
     DATA_RESULTS = 'results'
+    DATA_PRODUCT_LIST = 'search_product_list'
     DATA_QUERY = 'query'
     DATA_QUICK_JUMP = "quickjump"
+    DATA_QUERY_SUGGESTION = 'query_suggestion'
     DATA_SEARCH_EXTRAS = 'extra_search_fields'
 
     #bhsearch may support more pluggable views later
-    VIEWS_SUPPORTED = {
-        None: "Free text",
-        DATA_VIEW_GRID: "Grid"
-    }
+    VIEWS_SUPPORTED = (
+        (None, "Free text"),
+        (DATA_VIEW_GRID, "Grid"),
+    )
 
     VIEWS_WITH_KNOWN_FIELDS = [DATA_VIEW_GRID]
     OBLIGATORY_FIELDS_TO_SELECT = [IndexFields.ID, IndexFields.TYPE]
@@ -341,10 +424,17 @@ class RequestContext(object):
             default_view,
             all_grid_fields,
             default_facets,
+            global_quicksearch,
+            query_suggestions,
             ):
         self.env = env
         self.req = req
-        self.parameters = RequestParameters(req)
+        self.requires_redirect = False
+        self._handle_multiproduct_parameters(req, global_quicksearch)
+        self.parameters = RequestParameters(
+            req,
+            href=get_global_env(self.env).href
+        )
         self.data = {
             self.DATA_QUERY: self.parameters.query,
             self.DATA_SEARCH_EXTRAS: [],
@@ -356,16 +446,13 @@ class RequestContext(object):
         self.view = None
         self.page = self.parameters.page
         self.pagelen = self.parameters.pagelen
+        self.query_suggestions = query_suggestions
 
         if self.parameters.sort:
             self.sort = self.parameters.sort
-            self.data[self.DATA_SEARCH_EXTRAS].append(
-                (RequestParameters.SORT, self.parameters.sort_string)
-            )
         else:
             self.sort = self.DEFAULT_SORT
 
-
         self.allowed_participants, self.sorted_participants = \
             self._get_allowed_participants(req)
 
@@ -377,15 +464,30 @@ class RequestContext(object):
             self.active_type = None
             self.active_participant = None
 
-        self._prepare_allowed_types()
-        self._prepare_active_filter_queries()
+        self.active_product = self.parameters.product
+
+        self._prepare_active_type()
+        self._prepare_hidden_search_fields()
         self._prepare_quick_jump()
 
+        # Compatibility with trac search
+        self._process_legacy_type_filters(req, search_participants)
+        if not req.path_info.startswith(BHSEARCH_URL):
+            self.requires_redirect = True
+
         self.fields = self._prepare_fields_and_view()
         self.query_filter = self._prepare_query_filter()
         self.facets = self._prepare_facets()
 
+    def _handle_multiproduct_parameters(self, req, global_quicksearch):
+        if not using_multiproduct(self.env):
+            return
 
+        if self.env.parent is not None:
+            if not global_quicksearch:
+                req.args[RequestParameters.PRODUCT] = \
+                    self.env.product.prefix
+            self.requires_redirect = True
 
     def _get_allowed_participants(self, req):
         allowed_participants = {}
@@ -397,58 +499,34 @@ class RequestContext(object):
                 ordered_participants.append(participant)
         return allowed_participants, ordered_participants
 
-
-    def _prepare_allowed_types(self):
+    def _prepare_active_type(self):
         active_type = self.parameters.type
         if active_type and active_type not in self.allowed_participants:
             raise TracError(_("Unsupported resource type: '%(name)s'",
-                name=active_type))
-        allowed_types = [
-            dict(
-                label=_("All"),
-                active=(active_type is None),
-                href=self.parameters.create_href(
-                    skip_type=True,
-                    skip_page=True,
-                    force_filters=[],
-                ),
-            )
-        ]
-        #we want obtain the same order as in search participants options
-        for participant in self.sorted_participants:
-            allowed_types.append(dict(
-                label=_(participant.get_title()),
-                active=(participant.get_participant_type() == active_type),
-                href=self.parameters.create_href(
-                    type=participant.get_participant_type(),
-                    skip_page=True,
-                    force_filters=[],
-                ),
-            ))
-        self.data[self.DATA_TYPES] = allowed_types
-        self.data[self.DATA_SEARCH_EXTRAS].append(
-            (RequestParameters.TYPE, active_type)
-        )
+                            name=active_type))
 
-    def _prepare_active_filter_queries(self):
-        current_filters = self.parameters.filter_queries
-
-        def remove_filters_from_list(filer_to_cut_from):
-            return current_filters[:current_filters.index(filer_to_cut_from)]
-
-        active_filter_queries = [
-            dict(
-                href=self.parameters.create_href(
-                    force_filters=remove_filters_from_list(filter_query)
-                ),
-                label=filter_query,
-                query=filter_query,
-            ) for filter_query in self.parameters.filter_queries
-        ]
-        self.data[self.DATA_ACTIVE_FILTER_QUERIES] = active_filter_queries
-        for filter_query in active_filter_queries:
+    def _prepare_hidden_search_fields(self):
+        if self.active_type:
             self.data[self.DATA_SEARCH_EXTRAS].append(
-                (RequestParameters.FILTER_QUERY, filter_query['query'])
+                (RequestParameters.TYPE, self.active_type)
+            )
+
+        if self.parameters.product:
+            self.data[self.DATA_SEARCH_EXTRAS].append(
+                (RequestParameters.PRODUCT, self.parameters.product)
+            )
+
+        if self.parameters.view:
+            self.data[self.DATA_SEARCH_EXTRAS].append(
+                (RequestParameters.VIEW, self.parameters.view)
+            )
+        if self.parameters.sort:
+            self.data[self.DATA_SEARCH_EXTRAS].append(
+                (RequestParameters.SORT, self.parameters.sort_string)
+            )
+        for filter_query in self.parameters.filter_queries:
+            self.data[self.DATA_SEARCH_EXTRAS].append(
+                (RequestParameters.FILTER_QUERY, filter_query)
             )
 
     def _prepare_quick_jump(self):
@@ -480,7 +558,9 @@ class RequestContext(object):
                 description = link.attrib.get('title', '')
         if quickjump_href:
             # Only automatically redirect to local quickjump links
-            if not quickjump_href.startswith(req.base_path or '/'):
+            base_path = req.base_path.replace('@', '%40')
+            redirect_href = quickjump_href.replace('@', '%40')
+            if not redirect_href.startswith(base_path or '/'):
                 noquickjump = True
             if noquickjump:
                 return {'href': quickjump_href, 'name': tag.EM(name),
@@ -488,7 +568,6 @@ class RequestContext(object):
             else:
                 req.redirect(quickjump_href)
 
-
     def _prepare_fields_and_view(self):
         self._add_views_selector()
         self.view = self._get_view()
@@ -511,18 +590,14 @@ class RequestContext(object):
 
     def _add_views_selector(self):
         active_view = self.parameters.view
-        if active_view:
-            self.data[self.DATA_SEARCH_EXTRAS].append(
-                (RequestParameters.VIEW, active_view)
-            )
 
         all_views = []
-        for view, label in self.VIEWS_SUPPORTED.iteritems():
+        for view, label in self.VIEWS_SUPPORTED:
             all_views.append(dict(
                 label=_(label),
                 href=self.parameters.create_href(
                     view=view, skip_view=(view is None)),
-                is_active = (view == active_view)
+                is_active=(view == active_view)
             ))
         self.data[self.DATA_ALL_VIEWS] = all_views
 
@@ -575,6 +650,10 @@ class RequestContext(object):
             query_filters.append(
                 self._create_term_expression(
                     IndexFields.TYPE, self.active_type))
+        if self.active_product is not None:
+            query_filters.append(self._create_term_expression(
+                IndexFields.PRODUCT, self.active_product or None)
+            )
         return query_filters
 
     def _create_term_expression(self, field, field_value):
@@ -589,15 +668,37 @@ class RequestContext(object):
     def _prepare_facets(self):
         #TODO: add possibility of specifying facets in query parameters
         if self.active_participant:
-            facets =  self.active_participant.get_default_facets()
+            facets = self.active_participant.get_default_facets()
         else:
-            facets =  self.default_facets
+            facets = self.default_facets
         return facets
 
+    def _process_legacy_type_filters(self, req, search_participants):
+        legacy_type_filters = [sp.get_participant_type()
+                               for sp in search_participants
+                               if sp.get_participant_type() in req.args]
+        if legacy_type_filters:
+            params = self.parameters.params
+            if len(legacy_type_filters) == 1:
+                self.parameters.type = params[RequestParameters.TYPE] = \
+                    legacy_type_filters[0]
+            else:
+                filter_queries = self.parameters.filter_queries
+                if params[RequestParameters.FILTER_QUERY] is not filter_queries:
+                    params[RequestParameters.FILTER_QUERY] = filter_queries
+                filter_queries.append(
+                    'type:(%s)' % ' OR '.join(legacy_type_filters)
+                )
+            self.requires_redirect = True
+
     def _process_doc(self, doc):
         ui_doc = dict(doc)
-        ui_doc["href"] = self.req.href(doc['type'], doc['id'])
-        #todo: perform content adaptation here
+        if doc['product']:
+            product_href = ProductEnvironment(self.env, doc['product']).href
+            # pylint: disable=too-many-function-args
+            ui_doc["href"] = product_href(doc['type'], doc['id'])
+        else:
+            ui_doc["href"] = self.req.href(doc['type'], doc['id'])
 
         if doc['content']:
             ui_doc['content'] = shorten_result(doc['content'])
@@ -636,6 +737,7 @@ class RequestContext(object):
             add_link(self.req, 'prev', prev_href, _('Previous Page'))
 
         self.data[self.DATA_RESULTS] = results
+        prevnext_nav(self.req, _('Previous'), _('Next'))
 
     def _prepare_shown_pages(self, results):
         shown_pages = results.get_shown_pages(self.pagelen)
@@ -652,7 +754,8 @@ class RequestContext(object):
                                   query_result.highlighting)
         self._prepare_results(docs, query_result.hits)
         self._prepare_result_facet_counts(query_result.facets)
-        self._prepare_breadcrumbs_template()
+        self._prepare_breadcrumbs()
+        self._prepare_query_suggestion(query_result.query_suggestion)
         self.data[self.DATA_DEBUG] = query_result.debug
         if self.parameters.debug:
             self.data[self.DATA_DEBUG]['enabled'] = True
@@ -677,9 +780,13 @@ class RequestContext(object):
         }
 
         """
-        facet_counts = dict()
+        facet_counts = []
         if result_facets:
-            for field, facets_dict in result_facets.iteritems():
+            for field in self.facets:
+                if field == IndexFields.PRODUCT and \
+                        not using_multiproduct(self.env):
+                    continue
+                facets_dict = result_facets.get(field, {})
                 per_field_dict = dict()
                 for field_value, count in facets_dict.iteritems():
                     if field == IndexFields.TYPE:
@@ -687,6 +794,11 @@ class RequestContext(object):
                             skip_page=True,
                             force_filters=[],
                             type=field_value)
+                    elif field == IndexFields.PRODUCT:
+                        href = self.parameters.create_href(
+                            skip_page=True,
+                            product=field_value or u'',
+                        )
                     else:
                         href = self.parameters.create_href(
                             skip_page=True,
@@ -698,7 +810,7 @@ class RequestContext(object):
                         count=count,
                         href=href
                     )
-                facet_counts[_(field)] = per_field_dict
+                facet_counts.append((_(field), per_field_dict))
 
         self.data[self.DATA_FACET_COUNTS] = facet_counts
 
@@ -719,5 +831,79 @@ class RequestContext(object):
     def _create_genshi_fragment(self, html_fragment):
         return tag(HTML(html_fragment))
 
+    def _prepare_breadcrumbs(self):
+        self._prepare_breadcrumbs_template()
+        self._prepare_product_breadcrumb()
+        self._prepare_query_filter_breadcrumbs()
+
     def _prepare_breadcrumbs_template(self):
         self.data[self.DATA_BREADCRUMBS_TEMPLATE] = 'bhsearch_breadcrumbs.html'
+
+    def _prepare_product_breadcrumb(self):
+        if not using_multiproduct(self.env):
+            return
+        product_search = lambda x: self.parameters.create_href(product=x)
+        all_products_search = self.parameters.create_href(skip_product=True)
+
+        global_product = [(u'', _(u'Global product'), product_search(u''))]
+        products = \
+            ProductModule.get_product_list(self.env, self.req, product_search)
+        all_products = [(None, _(u'All products'), all_products_search)]
+
+        search_product_list = global_product + products + all_products
+
+        # pylint: disable=unused-variable
+        for prefix, name, url in search_product_list:
+            if prefix == self.active_product:
+                self.data[self.DATA_ACTIVE_PRODUCT] = name
+                break
+        else:
+            self.data[self.DATA_ACTIVE_PRODUCT] = self.active_product
+        self.data[self.DATA_PRODUCT_LIST] = search_product_list
+
+    def _prepare_query_filter_breadcrumbs(self):
+        current_filters = self.parameters.filter_queries
+
+        def remove_filter_from_list(filter_to_remove):
+            new_filters = list(current_filters)
+            new_filters.remove(filter_to_remove)
+            return new_filters
+
+        if self.active_type:
+            type_query = self._create_term_expression('type', self.active_type)
+            type_filters = [dict(
+                href=self.parameters.create_href(skip_type=True,
+                                                 force_filters=[]),
+                label=unicode(self.active_type).capitalize(),
+                query=type_query,
+            )]
+        else:
+            type_filters = []
+
+        active_filter_queries = [
+            dict(
+                href=self.parameters.create_href(
+                    force_filters=remove_filter_from_list(filter_query)
+                ),
+                label=filter_query,
+                query=filter_query,
+            ) for filter_query in self.parameters.filter_queries
+        ]
+        active_query = dict(
+            href=self.parameters.create_href(skip_query=True),
+            label=u'"%s"' % self.parameters.query,
+            query=self.parameters.query
+        )
+
+        self.data[self.DATA_ACTIVE_FILTER_QUERIES] = \
+            type_filters + active_filter_queries
+        self.data[self.DATA_ACTIVE_QUERY] = active_query
+
+    def _prepare_query_suggestion(self, suggestion):
+        if self.query_suggestions and suggestion is not None:
+            self.data[self.DATA_QUERY_SUGGESTION] = dict(
+                query=suggestion,
+                href=self.parameters.create_href(query=suggestion)
+            )
+        else:
+            self.data[self.DATA_QUERY_SUGGESTION] = None
