@@ -19,11 +19,14 @@
 #  under the License.
 
 r"""Whoosh specific backend for Bloodhound Search plugin."""
+from bhsearch import BHSEARCH_CONFIG_SECTION
 from bhsearch.api import ISearchBackend, DESC, QueryResult, SCORE, \
     IDocIndexPreprocessor, IResultPostprocessor, IndexFields, \
     IQueryPreprocessor
 import os
 from bhsearch.search_resources.ticket_search import TicketFields
+from bhsearch.security import SecurityPreprocessor
+from bhsearch.utils import get_global_env
 from trac.core import Component, implements, TracError
 from trac.config import Option, IntOption
 from trac.util.text import empty
@@ -32,21 +35,50 @@ from whoosh.fields import Schema, ID, DATETIME, KEYWORD, TEXT
 from whoosh import index, analysis
 import whoosh
 import whoosh.highlight
+from whoosh.collectors import FilterCollector
 from whoosh.writing import AsyncWriter
 from datetime import datetime
 
+from bhsearch.whoosh_fixes import fixes_for
+for fix in fixes_for(whoosh.__version__):
+    apply(fix)
+
 UNIQUE_ID = "unique_id"
 
+
 class WhooshBackend(Component):
     """
     Implements Whoosh SearchBackend interface
     """
     implements(ISearchBackend)
 
-    index_dir_setting = Option('bhsearch', 'whoosh_index_dir', 'whoosh_index',
-        """Relative path is resolved relatively to the
+    index_dir_setting = Option(
+        BHSEARCH_CONFIG_SECTION,
+        'whoosh_index_dir',
+        default='whoosh_index',
+        doc="""Relative path is resolved relatively to the
         directory of the environment.""")
 
+    advanced_security = Option(
+        BHSEARCH_CONFIG_SECTION,
+        'advanced_security',
+        default=False,
+        doc="Check view permission for each document when retrieving results."
+    )
+
+    max_fragment_size = IntOption(
+        BHSEARCH_CONFIG_SECTION,
+        'max_fragment_size',
+        default=240,
+        doc="The maximum number of characters allowed in a fragment.")
+
+    fragment_surround = IntOption(
+        BHSEARCH_CONFIG_SECTION,
+        'fragment_surround',
+        default=60,
+        doc="""The number of extra characters of context to add both before
+        the first matched term and after the last matched term.""")
+
     #This is schema prototype. It will be changed later
     #TODO: add other fields support, add dynamic field support.
     #Schema must be driven by index participants
@@ -71,22 +103,29 @@ class WhooshBackend(Component):
         changes=TEXT(analyzer=analysis.StandardAnalyzer(stoplist=None)),
         owner=TEXT(stored=True,
                    analyzer=analysis.SimpleAnalyzer()),
+        repository=TEXT(stored=True,
+                        analyzer=analysis.SimpleAnalyzer()),
+        revision=TEXT(stored=True,
+                      analyzer=analysis.SimpleAnalyzer()),
+        message=TEXT(stored=True,
+                     analyzer=analysis.SimpleAnalyzer()),
+        required_permission=ID(),
+        name=TEXT(stored=True,
+                  analyzer=analysis.SimpleAnalyzer()),
+        query_suggestion_basket=TEXT(analyzer=analysis.SimpleAnalyzer(),
+                                     spelling=True),
+        relations=KEYWORD(lowercase=True, commas=True),
     )
 
-    max_fragment_size = IntOption('bhsearch', 'max_fragment_size', 240,
-                               'The maximum number of characters allowed in a '
-                               'fragment.')
-
-    fragment_surround = IntOption('bhsearch', 'fragment_surround', 60,
-                               'The number of extra characters of context '
-                               'to add both before the first matched term '
-                               'and after the last matched term.')
-
     def __init__(self):
         self.index_dir = self.index_dir_setting
         if not os.path.isabs(self.index_dir):
-            self.index_dir = os.path.join(self.env.path, self.index_dir)
-        self.index = self._open_or_create_index_if_missing()
+            self.index_dir = os.path.join(get_global_env(self.env).path,
+                                          self.index_dir)
+        if index.exists_in(self.index_dir):
+            self.index = index.open_dir(self.index_dir)
+        else:
+            self.index = None
 
     #ISearchBackend methods
     def start_operation(self):
@@ -108,7 +147,9 @@ class WhooshBackend(Component):
             writer = self._create_writer()
 
         self._reformat_doc(doc)
-        doc[UNIQUE_ID] = self._create_unique_id(doc["type"], doc["id"])
+        doc[UNIQUE_ID] = self._create_unique_id(doc.get("product", ''),
+                                                doc["type"],
+                                                doc["id"])
         self.log.debug("Doc to index: %s", doc)
         try:
             writer.update_document(**doc)
@@ -133,8 +174,8 @@ class WhooshBackend(Component):
             else:
                 doc[key] = self._to_whoosh_format(value)
 
-    def delete_doc(self, doc_type, doc_id, operation_context=None):
-        unique_id = self._create_unique_id(doc_type, doc_id)
+    def delete_doc(self, product, doc_type, doc_id, operation_context=None):
+        unique_id = self._create_unique_id(product, doc_type, doc_id)
         self.log.debug('Removing document from the index: %s', unique_id)
         writer = operation_context
         is_local_writer = False
@@ -156,7 +197,7 @@ class WhooshBackend(Component):
         writer.commit(optimize=True)
 
     def is_index_outdated(self):
-        return not self.index.schema == self.SCHEMA
+        return self.index is None or not self.index.schema == self.SCHEMA
 
     def recreate_index(self):
         self.log.info('Creating Whoosh index in %s' % self.index_dir)
@@ -164,14 +205,9 @@ class WhooshBackend(Component):
         self.index = index.create_in(self.index_dir, schema=self.SCHEMA)
         return self.index
 
-    def _open_or_create_index_if_missing(self):
-        if index.exists_in(self.index_dir):
-            return index.open_dir(self.index_dir)
-        else:
-            return self.recreate_index()
-
     def query(self,
               query,
+              query_string=None,
               sort = None,
               fields = None,
               filter = None,
@@ -179,21 +215,12 @@ class WhooshBackend(Component):
               pagenum = 1,
               pagelen = 20,
               highlight = False,
-              highlight_fields = None):
-        """
-        Perform query.
-
-        Temporary fixes:
-        Whoosh 2.4 raises an error when simultaneously using filters and facets
-        in search:
-            AttributeError: 'FacetCollector' object has no attribute 'offset'
-        The problem should be fixed in the next release. For more info read
-        https://bitbucket.org/mchaput/whoosh/issue/274
-        A workaround is introduced to join query and filter in query and not
-        using filter parameter. Remove the workaround when the fixed version
-        of Whoosh is applied.
-        """
+              highlight_fields = None,
+              context=None):
+        # pylint: disable=too-many-locals
         with self.index.searcher() as searcher:
+            self._apply_advanced_security(searcher, context)
+
             highlight_fields = self._prepare_highlight_fields(highlight,
                                                               highlight_fields)
 
@@ -204,11 +231,6 @@ class WhooshBackend(Component):
             #groupedby = self._prepare_groupedby(facets)
             groupedby = facets
 
-            #workaround of Whoosh bug, read method __doc__
-            query = self._workaround_join_query_and_filter(
-                query,
-                filter)
-
             query_parameters = dict(
                 query = query,
                 pagenum = pagenum,
@@ -216,8 +238,7 @@ class WhooshBackend(Component):
                 sortedby = sortedby,
                 groupedby = groupedby,
                 maptype=whoosh.sorting.Count,
-                #workaround of Whoosh bug, read method __doc__
-                #filter = filter,
+                filter = filter,
             )
             self.env.log.debug("Whoosh query to execute: %s",
                 query_parameters)
@@ -226,24 +247,61 @@ class WhooshBackend(Component):
                                             fields,
                                             highlight_fields,
                                             query_parameters)
+            if query_string is not None:
+                c = searcher.correct_query(query, query_string)
+                results.query_suggestion = c.string
             try:
                 actual_query = unicode(query.simplify(searcher))
                 results.debug['actual_query'] = actual_query
-            except TypeError:
+            # pylint: disable=bare-except
+            except:
                 # Simplify has a bug that causes it to fail sometimes.
                 pass
         return results
 
-    def _workaround_join_query_and_filter(
-            self,
-            query_expression,
-            query_filter):
-        if not query_filter:
-            return query_expression
-        return whoosh.query.And((query_expression, query_filter))
+    def _apply_advanced_security(self, searcher, context=None):
+        if not self.advanced_security:
+            return
 
-    def _create_unique_id(self, doc_type, doc_id):
-        return u"%s:%s" % (doc_type, doc_id)
+        old_collector = searcher.collector
+        security_processor = SecurityPreprocessor(self.env)
+
+        def check_permission(doc):
+            return security_processor.check_permission(doc, context)
+
+        def collector(*args, **kwargs):
+            c = old_collector(*args, **kwargs)
+            if isinstance(c, FilterCollector):
+                c = AdvancedFilterCollector(
+                    c.child, c.allow, c.restrict, check_permission
+                )
+            else:
+                c = AdvancedFilterCollector(
+                    c, None, None, check_permission
+                )
+            return c
+        searcher.collector = collector
+
+    def _create_unique_id(self, product, doc_type, doc_id):
+        product, doc_type, doc_id = \
+            self._apply_empty_facets_workaround(product, doc_type, doc_id)
+
+        if product:
+            return u"%s:%s:%s" % (product, doc_type, doc_id)
+        else:
+            return u"%s:%s" % (doc_type, doc_id)
+
+    def _apply_empty_facets_workaround(self, product, doc_type, doc_id):
+        # Apply the same workaround that is used at insertion time
+        doc = {
+            IndexFields.PRODUCT: product,
+            IndexFields.TYPE: doc_type,
+            IndexFields.ID: doc_id,
+        }
+        WhooshEmptyFacetErrorWorkaround(self.env).pre_process(doc)
+        return (doc[IndexFields.PRODUCT],
+                doc[IndexFields.TYPE],
+                doc[IndexFields.ID])
 
     def _to_whoosh_format(self, value):
         if isinstance(value, basestring):
@@ -437,6 +495,7 @@ class WhooshEmptyFacetErrorWorkaround(Component):
         IndexFields.STATUS,
         TicketFields.MILESTONE,
         TicketFields.COMPONENT,
+        IndexFields.PRODUCT,
     ]
 
     #IDocIndexPreprocessor methods
@@ -462,7 +521,7 @@ class WhooshEmptyFacetErrorWorkaround(Component):
                     del doc[field]
 
     #IQueryPreprocessor methods
-    def query_pre_process(self, query_parameters):
+    def query_pre_process(self, query_parameters, context=None):
         """
         Go through filter queries and replace "NOT (field_name:*)" query with
         "field_name:NULL_MARKER" query.
@@ -470,16 +529,23 @@ class WhooshEmptyFacetErrorWorkaround(Component):
         This is really quick fix to make prototype working with hope that
         the next Whoosh version will be released soon.
         """
+        # pylint: disable=unused-argument
         if "filter" in query_parameters and query_parameters["filter"]:
-            self._find_and_fix_condition(query_parameters["filter"])
+            term_to_replace = \
+                self._find_and_fix_condition(query_parameters["filter"])
+            if term_to_replace:
+                query_parameters["filter"] = term_to_replace
         if "query" in query_parameters and query_parameters["query"]:
-            self._find_and_fix_condition(query_parameters["query"])
+            term_to_replace = \
+                self._find_and_fix_condition(query_parameters["query"])
+            if term_to_replace:
+                query_parameters["query"] = term_to_replace
 
     def _find_and_fix_condition(self, filter_condition):
         if isinstance(filter_condition, whoosh.query.CompoundQuery):
             sub_queries = list(filter_condition.subqueries)
             for i, subquery in enumerate(sub_queries):
-                term_to_replace =  self._find_and_fix_condition(subquery)
+                term_to_replace = self._find_and_fix_condition(subquery)
                 if term_to_replace:
                     filter_condition.subqueries[i] = term_to_replace
         elif isinstance(filter_condition, whoosh.query.Not):
@@ -488,3 +554,44 @@ class WhooshEmptyFacetErrorWorkaround(Component):
                not_query.fieldname in self.should_not_be_empty_fields:
                 return whoosh.query.Term(not_query.fieldname, self.NULL_MARKER)
         return None
+
+
+class AdvancedFilterCollector(FilterCollector):
+    """An advanced filter collector, accepting a callback function that
+    will be called for each document to determine whether it should be
+    filtered out or not.
+
+    Please note that it can be slow. Very slow.
+    """
+
+    def __init__(self, child, allow, restrict, filter_func=None):
+        FilterCollector.__init__(self, child, allow, restrict)
+        self.filter_func = filter_func
+
+    def collect_matches(self):
+        child = self.child
+        _allow = self._allow
+        _restrict = self._restrict
+
+        if _allow is not None or _restrict is not None:
+            filtered_count = self.filtered_count
+            for sub_docnum in child.matches():
+                global_docnum = self.offset + sub_docnum
+                if ((_allow is not None and global_docnum not in _allow)
+                    or (_restrict is not None and global_docnum in _restrict)):
+                    filtered_count += 1
+                    continue
+
+                if self.filter_func:
+                    doc = self.subsearcher.stored_fields(sub_docnum)
+                    if not self.filter_func(doc):
+                        filtered_count += 1
+                        continue
+
+                child.collect(sub_docnum)
+            # pylint: disable=attribute-defined-outside-init
+            self.filtered_count = filtered_count
+        else:
+            # If there was no allow or restrict set, don't do anything special,
+            # just forward the call to the child collector
+            child.collect_matches()
