@@ -26,7 +26,7 @@ from genshi.output import DocType
 from trac.config import ListOption, Option
 from trac.core import Component, TracError, implements
 from trac.mimeview.api import get_mimetype
-from trac.resource import Resource
+from trac.resource import get_resource_url, Neighborhood, Resource
 from trac.ticket.model import Ticket, Milestone
 from trac.ticket.notification import TicketNotifyEmail
 from trac.ticket.web_ui import TicketModule
@@ -398,9 +398,15 @@ class BloodhoundTheme(ThemeBase):
 
     def _modify_admin_breadcrumb(self, req, template, data, content_type, is_active):
         # override 'normal' product list with the admin one
-        glsettings = (None, _('(Global settings)'), req.href.admin())
-        admin_url = lambda x: req.href.products(x, 'admin')
-        data['admin_product_list'] = [glsettings] + \
+
+        def admin_url(prefix):
+            env = ProductEnvironment.lookup_env(self.env, prefix)
+            href = ProductEnvironment.resolve_href(env, self.env)
+            return href.admin()
+
+        global_settings = (None, _('(Global settings)'), admin_url(None))
+
+        data['admin_product_list'] = [global_settings] + \
             ProductModule.get_product_list(self.env, req, admin_url)
 
         if isinstance(req.perm.env, ProductEnvironment):
@@ -409,7 +415,7 @@ class BloodhoundTheme(ThemeBase):
                 (product.prefix, product.name,
                  req.href.products(product.prefix, 'admin'))
         else:
-            data['admin_current_product'] = glsettings
+            data['admin_current_product'] = global_settings
         data['resourcepath_template'] = 'bh_path_general.html'
 
     def _modify_browser(self, req, template, data, content_type, is_active):
@@ -424,7 +430,8 @@ class BloodhoundTheme(ThemeBase):
 
     def _add_products_general_breadcrumb(self, req, template, data,
                                          content_type, is_active):
-        data['resourcepath_template'] = 'bh_path_general.html'
+        if isinstance(req.perm.env, ProductEnvironment):
+            data['resourcepath_template'] = 'bh_path_general.html'
 
     # INavigationContributor methods
 
@@ -521,7 +528,10 @@ class QuickCreateTicketDialog(Component):
             self.log.exception("BH: Quick create ticket failed %s" % (exc,))
             req.send(str(exc), 'plain/text', 500)
         else:
-            req.send(to_json({'product': product, 'id': tid}),
+            tres = Neighborhood('product', product)('ticket', tid)
+            href = req.href
+            req.send(to_json({'product': product, 'id': tid,
+                              'url': get_resource_url(self.env, tres, href)}),
                      'application/json')
 
     def _get_ticket_module(self):
