@@ -18,12 +18,13 @@ import copy
 import re
 
 from genshi.builder import tag
+from genshi.core import Markup, unescape
 
 from trac.cache import cached
 from trac.config import *
 from trac.core import *
 from trac.perm import IPermissionRequestor, PermissionCache, PermissionSystem
-from trac.resource import IResourceManager
+from trac.resource import IResourceManager, ResourceSystem
 from trac.util import Ranges, as_int
 from trac.util.text import shorten_line
 from trac.util.translation import _, N_, gettext
@@ -186,6 +187,16 @@ class ITicketFieldProvider(Interface):
         specified as optional.
         """
 
+    def get_raw_fields():
+        """Returns a list of fields, each represents ticket field
+        dictionary. For example:
+            * name: field name
+            * type: field type
+            * label: the label to display, preferably wrapped with N_()
+            * format: field format
+            * other appropriate field properties
+        """
+
 class TicketSystem(Component):
     implements(IPermissionRequestor, IWikiSyntaxProvider, IResourceManager,
                ITicketFieldProvider)
@@ -385,12 +396,33 @@ class TicketSystem(Component):
             field['custom'] = True
             fields.append(field)
 
+        #TODO: this is Bloodhound specific patch to the Trac. Contact Trac
+        # community about possibility to apply the change to the Trac codebase
+        self._add_raw_fields_from_field_providers(fields)
+
         return fields
 
     reserved_field_names = ['report', 'order', 'desc', 'group', 'groupdesc',
                             'col', 'row', 'format', 'max', 'page', 'verbose',
                             'comment', 'or']
 
+    def _add_raw_fields_from_field_providers(self, fields):
+        for field_provider in self.ticket_field_providers:
+            if hasattr(field_provider, 'get_raw_fields'):
+                raw_fields = field_provider.get_raw_fields()
+                if raw_fields:
+                    for raw_field in raw_fields:
+                        self._add_raw_field(
+                            raw_field, fields)
+
+    def _add_raw_field(self, raw_field, fields):
+        if raw_field["name"] in [f['name'] for f in fields]:
+            self.log.warning(
+                'Duplicate field name "%s" (ignoring)', raw_field["name"])
+        else:
+            fields.append(raw_field)
+
+
     def get_custom_fields(self):
         return copy.deepcopy(self.custom_fields)
 
@@ -508,7 +540,11 @@ class TicketSystem(Component):
                 ranges = str(r)
                 if params:
                     params = '&' + params[1:]
-                label_wrap = label.replace(',', u',\u200b')
+                if isinstance(label, Markup):
+                    _label = unescape(label)
+                else:
+                    _label = label
+                label_wrap = _label.replace(',', u',\u200b')
                 ranges_wrap = ranges.replace(',', u', ')
                 return tag.a(label_wrap,
                              title=_("Tickets %(ranges)s", ranges=ranges_wrap),
@@ -552,15 +588,17 @@ class TicketSystem(Component):
 
     def get_resource_description(self, resource, format=None, context=None,
                                  **kwargs):
+        nbhprefix = ResourceSystem(self.env).neighborhood_prefix(
+                resource.neighborhood)
         if format == 'compact':
-            return '#%s' % resource.id
+            return '%s#%s' % (nbhprefix, resource.id)
         elif format == 'summary':
             from trac.ticket.model import Ticket
             ticket = Ticket(self.env, resource.id)
             args = [ticket[f] for f in ('summary', 'status', 'resolution',
                                         'type')]
             return self.format_summary(*args)
-        return _("Ticket #%(shortname)s", shortname=resource.id)
+        return nbhprefix + _("Ticket #%(shortname)s", shortname=resource.id)
 
     def format_summary(self, summary, status=None, resolution=None, type=None):
         summary = shorten_line(summary)
